import requests
import urllib.parse

class DblpAPI():

    BASE_URL = 'https://dblp.org/search/{endpoint}/api?'

    @staticmethod
    def load_hits(endpoint: str, q: str, max_results: int = 100):
        hits = []
        total = -1
        first_id = 0

        while len(hits) != total and len(hits) < max_results:
            params = {
                'q': q,
                'format': 'json',
                'h': 50,
                'f': first_id
            }
            url = DblpAPI.BASE_URL.format(endpoint=endpoint) + urllib.parse.urlencode(params)
            re = requests.get(url=url)

            if not re.ok:
                raise DblpApiError(f'Could not load data from DBLP API. Status code: {re.status_code}. Error code: {re.json().get("status", "")}' )

            result = re.json()['result']

            total = int(result['hits']['@total'])
            sent = int(result['hits']['@sent'])
            if sent > 0:
                hits += result['hits']['hit']

            first_id = len(hits)

        return hits[:max_results]

class DblpApiError(Exception):
    pass




from dblpy.dblp_api import DblpAPI


class Venue():
    venue: str = ''
    acronym: str = ''
    type: str = ''
    url: str = ''

    def __init__(self, venue: str, acronym: str, type: str, url: str) -> None:
        self.venue = venue
        self.acronym = acronym
        self.type = type
        self.url = url

    def __str__(self) -> str:
        s = f'{self.venue} ({self.url})'
        return s

def get_venues(q: str, max_results: int = 100):
    venue_dict = DblpAPI.load_hits('venue', q=q, max_results=max_results)

    venue_objects = [
        Venue(
            venue=venue['info'].get('venue', ''),
            acronym=venue['info'].get('acronym', ''),
            type=venue['info'].get('type', ''),
            url=venue['info'].get('url', '')
        )
        for venue in venue_dict
    ]

    return venue_objects

if __name__ == "__main__":
    venues = get_venues(q='Berlin')
    for v in venues:
        print(v)



from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='dblpy-lib',
    packages=find_packages(include=['dblpy']),
    version='0.1.0',
    description='A wrapper for dblp.org search api',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Benedikt Maria Beckermann',
    author_email='hello@benedikt.click',
    url='https://github.com/greenBene/dblpy',
    license='CC0 1.0 Universal',
    install_requires=['requests~=2.27.1'],
    test_suite='tests'
)


from typing import List

from dblpy.dblp_api import DblpAPI

class AuthorNote():
    type:str = ''
    text:str = ''

    def __init__(self, type:str, text:str) -> None:
        self.type = type
        self.text = text

    def __str__(self) -> str:
        s = f'{self.type}: {self.text}'
        return s

class Author():
    name:str = ''
    notes:List[AuthorNote] = []
    url:str = ''
    aliases:List[str] = []

    def __init__(self, name:str, url:str, notes = None, aliases:dict = None) -> None:
        self.name = name
        self.url = url
        if notes:
            self._prepare_notes(notes)
        if aliases:
            self._prepare_aliases(aliases)

    def __str__(self) -> str:
        s = f'{self.name} ({self.url})'
        return s

    def _prepare_notes(self, notes):
        if type(notes) == dict:
            notes = [notes]
        self.notes = [
            AuthorNote(
                type=note['@type'], 
                text=note['text']) 
            for note in notes]
        return notes

    def _prepare_aliases(self, aliases):
        if type(aliases) != list:
            aliases = [aliases]

        self.aliases = [
            alias
            for alias in aliases]
        return aliases


def get_authors(q: str, max_results: int = 100) -> List[Author]:
    authors_dict = DblpAPI.load_hits(endpoint='author', q=q, max_results=max_results)
    authors_objects = [
        Author(
            name=author['info']['author'], 
            url=author['info']['url'], 
            notes=author['info']['notes']['note'] 
                if 'notes' in author['info'] else None,
            aliases=author['info']['aliases']['alias']
                if 'aliases' in author['info'] else None)
        for author in authors_dict]

    return authors_objects


if __name__ == '__main__':
    publ = get_authors(q='Donald Ervin Knuth')
    for p in publ:
        print(p)


from typing import List
from dblpy.dblp_api import DblpAPI

class Publication():
    authors: List[str] = []
    title: str = ''
    venue: str = ''
    volume: str = ''
    number: str = ''
    pages: str = ''
    publisher: str = ''
    year: str = ''
    type: str = ''
    access: str = ''
    key: str = ''
    doi: str = ''
    ee: str = ''
    url: str = ''

    def __init__(self, authors:List, title: str, venue: str, volume :str, number: str, pages: str, publisher: str, year: str, type: str, access: str, key: str, doi: str, ee: str, url: str) -> None:
        self.title = title
        self.venue = venue
        self.volume = volume
        self.number = number
        self.pages = pages
        self.publisher = publisher
        self.year = year
        self.type = type
        self.access = access
        self.key = key
        self.doi = doi
        self.ee = ee
        self.url = url
        self.authors = self._prepare_authors(authors)
    
    def _prepare_authors(self, authors):
        if type(authors) != list:
            authors = [authors]
        return [author['text'] for author in authors]

    def __str__(self) -> str:
        s = f'{self.year}. {self.authors[0]} et al. "{self.title}". ({self.url})'
        return s


def get_publications(q: str, max_results: int = 100) -> List[Publication]:
    publications_dict = DblpAPI.load_hits(endpoint='publ', q=q, max_results=max_results)

    publications_objects = [
        Publication(
            authors=publication['info']['authors']['author'],
            title=publication['info']['title'],
            venue=publication['info'].get('venue', ''),
            volume=publication['info'].get('volume', ''),
            number=publication['info'].get('number', ''),
            pages=publication['info'].get('pages', ''),
            publisher=publication['info'].get('publisher', ''),
            year=publication['info'].get('year', ''),
            type=publication['info'].get('type', ''),
            access=publication['info'].get('access', ''),
            key=publication['info'].get('key', ''),
            doi=publication['info'].get('doi', ''),
            ee=publication['info'].get('ee', ''),
            url=publication['info'].get('url', '')
        )
        for publication in publications_dict
    ]

    return publications_objects


if __name__ == '__main__':
    publ = get_publications(q='mHealthAtlas')
    for p in publ:
        print(p)